# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require 'sqlite3'
require 'singleton'

module NihongoBenkyo

class Entry
    attr_accessor :id, :in_kanji, :readings, :meanings

    def initialize
        @entry = nil
        @in_kanji = []
        @readings = {}
        @meanings = {}
    end

    def add_in_kanji(in_kanji)
        @in_kanji << in_kanji unless @in_kanji.include?(in_kanji)    
    end

    def add_reading(type, reading)
        @readings[type] ||= []
        @readings[type] << reading unless @readings[type].include?(reading)
    end

    def readings
        readings = []
        @readings.each_key do |type|
            readings += @readings[type]
        end
        readings
    end

    def readings_by_type
        @readings.collect do |reading| 
            reading[1].join(", ")
        end
    end

    def roma_readings_by_type
        @readings.collect do |reading| 
            reading[1].collect do |r|
                r.to_roma
            end.join(", ")
        end
    end

    def add_meaning(lang, meaning)
        @meanings[lang] ||= []
        @meanings[lang] << meaning unless @meanings[lang].include?(meaning)
    end

    def meanings_by_lang
        @meanings.collect do |meaning| 
            meaning[1].join(", ")
        end
    end
end

class Word < Entry
    attr_accessor :important
    
    def initialize
        super
        @important = false
    end
end

class Kanji < Entry
    attr_accessor :strokes, :grade, :frequency
    
    def initialize
        super
        @strokes = []
        @grade = []
        @frequency = []
    end

    def to_s
        @in_kanji.first
    end

    def on_readings
        @readings["ja_on"] || []
    end

    def kun_readings
        @readings["ja_kun"] || []
    end
end

class Dictionary
    
    def self.exists?(dicname)
        db = Database.instance.database
        n = db.get_first_value("SELECT count(*) FROM dictionaries WHERE name = '#{dicname}'")
        n == "0" ? false : true
    end

    def self.list
        db = Database.instance.database
        db.execute("SELECT name, type, selected FROM dictionaries ORDER BY position ASC").collect do |row|
            [row[0], row[1].to_i, row[2].to_i]
        end
    end

    def self.remove(dicname)
        db = Database.instance.database
        did = db.get_first_value("SELECT id FROM dictionaries WHERE name = '#{dicname}'").to_i
        db.transaction
            entries = "(SELECT id FROM entries WHERE dictionary_id = #{did})"
            db.execute("DELETE FROM dictionaries WHERE id = #{did}")
            db.execute("DELETE FROM kanji WHERE entry_id IN #{entries}")
            db.execute("DELETE FROM readings WHERE entry_id IN #{entries}")
            db.execute("DELETE FROM translations WHERE entry_id IN #{entries}")   
            db.execute("DELETE FROM entries WHERE dictionary_id = #{did}")
        db.commit
        db.optimize
    end

    def self.change_position(old_path, new_path)
        db = Database.instance.database
        db.transaction
            db.execute("UPDATE dictionaries set position = -1 WHERE position = #{new_path}")
            db.execute("UPDATE dictionaries set position = #{new_path} WHERE position = #{old_path}")
            db.execute("UPDATE dictionaries set position = #{old_path} WHERE position = -1")
        db.commit
    end

    def self.has_wild_card?(word)
        (word.index("*") || word.index("?")).nil? ? false : true
    end

    def initialize(name)
        @dicname = name
        @db = Database.instance.database
    end    

    def selected=(selected)
        @db.execute("UPDATE dictionaries SET selected = #{selected} WHERE name = '#{@dicname}'")
    end
       
    def name=(newname)
        @db.execute("UPDATE dictionaries SET name = '#{newname}' WHERE name = '#{@dicname}'")
    end

    def type
        t = @db.get_first_value("SELECT type FROM dictionaries WHERE name = '#{@dicname}' LIMIT 1")
        t.to_i
    end

    def search_result_list(word) 
        wildcard = self.class.has_wild_card?(word)
        operator = wildcard ? 'GLOB' : '='

        if word.latin? 
            # from romaji or translation
            subquery = "SELECT entry_id FROM readings "
            if wildcard
                subquery += "WHERE reading GLOB '#{word.to_hira}' or reading GLOB '#{word.to_kata}'"
            else
                subquery += "WHERE reading IN('#{word.to_hira}', '#{word.to_kata}') "
            end
            subquery += "UNION "
            subquery += "SELECT entry_id FROM translations WHERE translation #{operator} '#{word}' "
        elsif word.kana?
            subquery = "SELECT entry_id FROM readings WHERE reading #{operator} '#{word}'"
        elsif word.japanese?
            subquery = "SELECT entry_id FROM kanji WHERE kanji #{operator} '#{word}'"
        else
            # everything because language has not been detected
            subquery = "SELECT entry_id FROM translations WHERE translation #{operator} '#{word}' "
            subquery += "UNION "
            subquery += "SELECT entry_id FROM readings WHERE reading #{operator} '#{word}' "
            subquery += "UNION "
            subquery += "SELECT entry_id FROM kanji WHERE kanji #{operator} '#{word}'"
        end 
        
        query = "SELECT e.id, k.kanji, r.reading, r.type, t.translation, t.language FROM entries e "
        query += "LEFT JOIN translations t ON t.entry_id = e.id "
        query += "LEFT JOIN kanji k ON k.entry_id = e.id "
        query += "LEFT JOIN readings r ON r.entry_id = e.id "
        query += "LEFT JOIN dictionaries d ON e.dictionary_id = d.id "
        query += "WHERE e.id IN (#{subquery}) AND d.selected = 1 "
        query += "AND d.name = '#{@dicname}' "
        query += "AND r.type IN('ja_on', 'ja_kun', '') "
        query += "ORDER BY d.position ASC"
                        
        current_entry = Entry.new # null entry
        results = []
        @db.execute(query) do |row|
            
            if current_entry.id != row[0].to_i
                # We are now in a new entry_id so unless we are in the first iteration,
                # we add the entry to the list of results
                unless current_entry.id.nil?
                    results << current_entry
                end
                # Need to restart current values
                current_entry = Entry.new
                current_entry.id = row[0].to_i
            end
            
            current_entry.add_in_kanji(row[1])
            current_entry.add_reading(row[3], row[2])
            current_entry.add_meaning(row[5], row[4])
        end
        
        # This is needed for the last entry that would have been forgotten without this
        unless current_entry.id.nil?
            results << current_entry
        end

        results
    end

    private

    def entry_id_in_kanji(entry_id)
        query = "SELECT kanji FROM kanji WHERE entry_id = #{entry_id}"
        @db.execute(query) do |row|
            yield row[0]
        end
    end

    def entry_id_meanings(entry_id)
        query = "SELECT language, translation FROM translations WHERE entry_id = #{entry_id}"
        @db.execute(query) do |row|
            yield *row
        end
    end

    def entry_id_readings(entry_id)
        query = "SELECT type, reading FROM readings WHERE entry_id = #{entry_id} "
        query += "AND type IN('ja_on', 'ja_kun', '')"

        @db.execute(query) do |row|
            yield *row
        end
    end
    
end

class WordDictionary < Dictionary

    def initialize(name)
        super
    end

    def search_entry(entry_id)
        word = Word.new
        word.id = entry_id

        word.important = important_entry?(entry_id)
    
        entry_id_in_kanji(entry_id) do |in_kanji|
            word.add_in_kanji(in_kanji)
        end

        entry_id_readings(entry_id) do |type, reading|
            word.add_reading(type, reading)
        end

        word
    end
    
    private

    def important_entry?(entry_id)
        # An entry is considered as important if its kanji or reading
        # has metadata values 'priority'
        kanji = "SELECT count(mv.id) FROM metadata_values mv "
        kanji += "LEFT JOIN metadata_names mn ON mn.id = mv.metadata_name_id "
        kanji += "WHERE mv.matching_id IN (SELECT id FROM kanji WHERE entry_id = #{entry_id}) "
        kanji += "AND mn.name = 'word_kanji_priority' "
        
        reading = "SELECT count(mv.id) FROM metadata_values mv "
        reading += "LEFT JOIN metadata_names mn ON mn.id = mv.metadata_name_id "
        reading += "WHERE mv.matching_id IN (SELECT id FROM readings WHERE entry_id = #{entry_id}) "
        reading += "AND mn.name = 'word_reading_priority' "
        
        sum = @db.get_first_value(kanji).to_i + @db.get_first_value(reading).to_i
        sum > 0 ? true : false
    end

end

class KanjiDictionary < Dictionary
    
    def initialize(name)
        super
    end

    def search_entry(entry_id)
        k = Kanji.new
        k.id = entry_id
        
        query = "SELECT mn.name, mv.value FROM metadata_values mv "
        query += "LEFT JOIN metadata_names mn ON mn.id = mv.metadata_name_id "
        query += "WHERE mv.matching_id = #{entry_id} "
        query += "AND mn.metadata_of = 'kanji_entry' "
        query += "AND mn.name IN ('kanji_frequency', 'kanji_grade', 'kanji_stroke_count') "
              
        @db.execute(query) do |row|
            k.strokes << row[1] if row[0] == "kanji_stroke_count"
            k.grade << row[1] if row[0] == "kanji_grade"
            k.frequency << row[1] if row[0] == "kanji_frequency"
        end

        entry_id_in_kanji(entry_id) do |in_kanji|
           k.add_in_kanji(in_kanji)
        end
        
        entry_id_meanings(entry_id) do |lang, meaning|
            k.add_meaning(lang, meaning)
        end
        
        entry_id_readings(entry_id) do |type, reading|
            k.add_reading(type, reading)
        end
        
        k
    end

end

class Database 
    include Singleton
        
    def initialize 
        open

        if tables_exist?
            @first_time = false
        else
            @first_time = true
            tables_create
        end
        
        # Says to sqlite to wait if the database is locked
        @db.busy_handler do |data, retries|
            sleep(1)
        end   
    end

    def first_time?
        @first_time
    end
    
    def open
        @db = SQLite3::Database.new(Config::DB_FILE)    
    end
    
    def close
        @db.close
    end   

    def database
        @db
    end
    
    def is_important_entry?(entry_id)
        # An entry is considered as important if its kanji or reading
        # has metadata values 'priority'
        kanji = "SELECT count(mv.id) FROM metadata_values mv "
        kanji += "LEFT JOIN metadata_names mn ON mn.id = mv.metadata_name_id "
        kanji += "WHERE mv.matching_id IN (SELECT id FROM kanji WHERE entry_id = #{entry_id}) "
        kanji += "AND mn.name = 'word_kanji_priority' "
        
        reading = "SELECT count(mv.id) FROM metadata_values mv "
        reading += "LEFT JOIN metadata_names mn ON mn.id = mv.metadata_name_id "
        reading += "WHERE mv.matching_id IN (SELECT id FROM readings WHERE entry_id = #{entry_id}) "
        reading += "AND mn.name = 'word_reading_priority' "
        
        sum = @db.get_first_value(kanji).to_i + @db.get_first_value(reading).to_i
        sum > 0 ? true : false
    end
    
    def search_entry_title(entry_id)
        kanji = "SELECT kanji FROM kanji WHERE entry_id = #{entry_id}"
        readings = "SELECT reading FROM readings WHERE entry_id = #{entry_id}"
        
        rows = @db.execute(kanji)
        rows = @db.execute(readings) unless rows.length > 0
        
        rows.collect! { |row| row[0] }
        rows.join(", ")
    end
    
    def search_metadata_values(matching_id, metadata_of, *metadata)
        metadata = metadata.collect { |m| "'#{m}'" }.join(", ")
        
        query = "SELECT mn.description, mv.value FROM metadata_values mv "
        query += "LEFT JOIN metadata_names mn ON mn.id = mv.metadata_name_id "
        query += "WHERE mv.matching_id = #{matching_id} "
        query += "AND mn.metadata_of = '#{metadata_of}' "
        query += "AND mn.name IN (#{metadata}) "
        
        hash = {}
        @db.execute(query) do |row|
            if hash.has_key?(row[0])
                hash[row[0]] << row[1]
            else
                hash[row[0]] = [row[1]]
            end    
        end
        hash
    end
    
    # Returns something like, for example :
    # dwarf => Antonyms => 巨人
    #       => Readings to which the translation is restricted => こびと
    # small person, child => Antonyms => 大人
    #                     => Part of speech => noun (common) (futsuumeishi)
    # [...]
    def search_metadata(entry_id, metadata_of, *metadata)
        query = case metadata_of
            when "word_kanji"
                "SELECT id, kanji FROM kanji WHERE entry_id = #{entry_id}"
            when "word_reading"
                "SELECT id, reading FROM readings WHERE entry_id = #{entry_id}"
            when "word_sense"
                "SELECT sense_id, translation FROM translations WHERE entry_id = #{entry_id} ORDER BY sense_id"
        end
        
        if metadata_of == "word_sense"
            hash = {}
            @db.execute(query) do |row|
                if hash.has_key?(row[0])
                    hash[row[0]] << row[1]
                else
                    hash[row[0]] = [row[1]]
                end
            end
            rows = hash.to_a.collect { |array| [array[0], array[1].join(", ")] }
            
        elsif metadata_of == "word_entry"
            rows = [[entry_id, ""]]
        else
            rows = @db.execute(query)
        end
        
        hash = {}
        rows.each do |row|
            id, title = row
            res_hash = search_metadata_values(id, metadata_of, *metadata)
            if metadata_of == "word_entry"
                hash[title] = res_hash if res_hash.length > 0
            else
                hash[title] = res_hash 
            end
        end
        hash     
    end
              
    def has_dictionaries?
        n = @db.get_first_value("SELECT count(*) FROM dictionaries")
        n == "0" ? false : true        
    end
       
    private
    
    def tables_exist?
        n = @db.get_first_value("SELECT count(*) FROM sqlite_master WHERE type = 'table' and name = 'entries'")
        n == "0" ? false : true
    end
    
    def tables_create
        path = File.join(NihongoBenkyo::Config::DATA_DIR, "sql", "db.sql")
        file = File.open(path, "r")
        @db.execute_batch(file.readlines.join)
        file.close
    end  

    def optimize
        @db.execute("VACUUM")
    end

end

end
