# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo
module UI

    class AddDictionaryDialog < Gtk::FileChooserDialog
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")
        
        def initialize(parent)
            super()
            self.title = _("Add a dictionary")
            self.action = Gtk::FileChooser::ACTION_OPEN
            self.transient_for = parent.preferences_dialog
            self.modal = true
            @parent = parent
            @db = Database.instance
            self.current_folder = ENV['HOME']     
           
            add_button(Gtk::Stock::CANCEL, Gtk::Dialog::RESPONSE_CANCEL)
            add_button = add_button(Gtk::Stock::APPLY, 
                                       Gtk::Dialog::RESPONSE_ACCEPT)
            add_button.sensitive = false
                 
            self.signal_connect('destroy') { hide }
            
            vbox = Gtk::VBox.new
            vbox.spacing = 15
            
            # Addding dic name to the file chooser
            
            hbox = Gtk::HBox.new
            name_entry = Gtk::Entry.new
            
            name_entry.signal_connect('changed') do
                add_button.sensitive = !name_entry.text.strip.empty?
            end
            
            name_entry.signal_connect("activate") do
                add_button.clicked
            end
            
            name_label = Gtk::Label.new(_("Dictionary name:"), true)
            name_label.xalign = 0
            name_label.mnemonic_widget = name_entry
            hbox.pack_start(name_label)
            hbox.pack_start(name_entry)
            hbox.show_all
            vbox.pack_start(hbox)
            
            # Adding dic type to the file chooser
            
            hbox = Gtk::HBox.new
            type_combo = Gtk::ComboBox.new
                        
            NihongoBenkyo::SUPPORTED_FORMATS.each { |s| type_combo.append_text(s) }
            type_combo.active = 0
            name_label = Gtk::Label.new(_("Dictionary type:"), true)
            name_label.xalign = 0
            name_label.mnemonic_widget = type_combo
            hbox.pack_start(name_label)
            hbox.pack_start(type_combo)
            hbox.show_all
            vbox.pack_start(hbox)
            
            # Adding langs to the file chooser
            
            jmdict_hbox, jmdict_checkbuttons = set_langs_hbox(NihongoBenkyo::JMDICT_LANGUAGES)
            kanjidic2_hbox, kanjidic2_checkbuttons = set_langs_hbox(NihongoBenkyo::KANJIDIC2_LANGUAGES)
            
            type_combo.signal_connect("changed") do
                selected = NihongoBenkyo::SUPPORTED_FORMATS[type_combo.active]
                if selected == "JMDict"
                    jmdict_hbox.visible = true
                    kanjidic2_hbox.visible = false
                elsif selected == "Kanjidic2"
                    kanjidic2_hbox.visible = true
                    jmdict_hbox.visible = false
                else
                    jmdict_hbox.visible = false
                    kanjidic2_hbox.visible = false
                end
            end
            
            vbox.pack_start(jmdict_hbox)
            vbox.pack_start(kanjidic2_hbox)
                     
            self.extra_widget = vbox
            
            # When "Add" is clicked         
            while run == Gtk::Dialog::RESPONSE_ACCEPT
                selected_format = NihongoBenkyo::SUPPORTED_FORMATS[type_combo.active]   
                    
                # Get an array of lang codes 
                langs_checkbuttons = if selected_format == "Kanjidic2"
                    kanjidic2_checkbuttons
                else
                    jmdict_checkbuttons
                end
                           
                langs = langs_checkbuttons.select do |lang, checkbutton| 
                    checkbutton.active?
                end 
                langs.collect! { |x| x[0] }
                
                
                    
                if name_entry.text.empty?
                    ErrorDialog.new(self, _("A dictionary name must be specified"))
                elsif Dictionary.exists?(name_entry.text)
                    ErrorDialog.new(self, _("Dictionary name already exists"))
                elsif langs.empty? and 
                    (selected_format == "JMDict" or selected_format == "Kanjidic2")
                    ErrorDialog.new(self, _("At least one language must be chosen"))
                else 
                    
                    self.sensitive = false
                    # Two processes cannot use sqlite at the same time!
                    @db.close
                    
                    pid = Process.fork do
                        begin 
                            case selected_format 
                                when "Edict"
                                    NihongoBenkyo::Import.edict(Config::DB_FILE, self.filename, name_entry.text)
                                when "JMnedict"
                                    NihongoBenkyo::Import.jmnedict(Config::DB_FILE, self.filename, name_entry.text)
                                when "JMDict"
                                    NihongoBenkyo::Import.jmdict(Config::DB_FILE, self.filename, 
                                                                name_entry.text, *langs)
                                when "Kanjidic2"
                                    NihongoBenkyo::Import.kanjidic2(Config::DB_FILE, self.filename, 
                                                                name_entry.text, *langs)
                            end
                        rescue NihongoBenkyo::Import::DicTypeException    
                        ensure
                            Process.kill(9, Process.pid)
                        end
                    end   
                    
                    thread = Thread.start { Process.waitpid(pid) }
                    
                    import_was_cancelled = false
                    pbar = ProgressBar.new(_("Adding dictionary..."), self) do
                        # Block called on cancel
                        Process.kill(9, pid)
                        thread.kill
                        import_was_cancelled = true
                    end 
                    
                    pbar.signal_connect("destroy") { thread.kill }
                    
                    while thread.alive?
                        pbar.pulse
                        Gtk.main_iteration while(Gtk.events_pending?)
                        sleep 1
                    end
                    
                    pbar.destroy unless pbar.destroyed?
                    
                    # Done !
                    self.sensitive = true
                    @db.open
                    
                    unless import_was_cancelled 
                        if Dictionary.exists?(name_entry.text)
                            @parent.append_dictionary(name_entry.text)
                        else 
                            ErrorDialog.new(self, _("Could not import dictionary"))
                        end
                    end
                    
                    break                                 
                end
            end
            
            self.destroy
        end    
        
        def set_langs_hbox(langs_array)
            langs_checkbuttons = {}
            table = Gtk::Table.new(1, langs_array.length, true)
            
            n = 0
            langs_array.each do |lang|
                langs_checkbuttons[lang] = Gtk::CheckButton.new(lang)
                langs_checkbuttons[lang].active = true
                table.attach_defaults(langs_checkbuttons[lang], n, n+1, 0, 1)
                n += 1
            end
            
            table.show_all
            table.visible = false
            
            langs_hbox = Gtk::HBox.new
            name_label = Gtk::Label.new(_("Languages to import:"), true)
            name_label.xalign = 0
            name_label.mnemonic_widget = table
            langs_hbox.pack_start(name_label)
            langs_hbox.pack_start(table)
            langs_hbox.show_all
            langs_hbox.visible = false
            
            return langs_hbox, langs_checkbuttons
        end
    
    end

end
end