# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo
module UI

    module Column
        ENTRY_ID, KANJI, READINGS, ROMAJI, MEANINGS = (0..4).to_a
        SEARCH_HASH = 0
    end
    
    module DicType
        WORD, KANJI = (0..1).to_a
    end
    
    module ResultType
        LIST, DETAILS = (0..1).to_a
    end

    class HistoryListView < Gtk::TreeView
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")
        
        def initialize
            super()
            self.model = Gtk::ListStore.new(Hash)
            self.headers_visible = false
            self.rules_hint = true
            
            @prefs = Preferences.instance
            
            # Column for the result type           
            renderer = Gtk::CellRendererPixbuf.new
            col = Gtk::TreeViewColumn.new("Result type", renderer)
            
            col.set_cell_data_func(renderer) do |col, renderer, model, iter| 
                case iter[Column::SEARCH_HASH][:res_type]
                    when ResultType::LIST
                        renderer.stock_id = Icon::RESULT_LIST
                    when ResultType::DETAILS
                        renderer.stock_id = Icon::RESULT_DETAILS
                end
            end
            col.min_width = 25
            self.append_column(col)
            
            # Column for the word
            renderer = Gtk::CellRendererText.new
            col = Gtk::TreeViewColumn.new("Searched word", renderer)
            
            col.set_cell_data_func(renderer) do |col, renderer, model, iter|
                text = iter[Column::SEARCH_HASH][:word]
                text = text.utf8_slice(0..7) + "..." if text.utf8_length > 10
                renderer.text = text
            end
            self.append_column(col)
            
            # Resize the history when a row is deleted
            self.model.signal_connect("row-deleted") { self.columns_autosize }
                
        end   
        
        def selected_search
            self.selected_iter[Column::SEARCH_HASH]
        end
        
        def get_search_iter(search)
            self.model.each do |model, path, iter|
                return iter if iter[Column::SEARCH_HASH] == search
            end    
            return nil
        end 
               
        def prepend_search(search)
            search_iter = self.model.prepend()
            search_iter[Column::SEARCH_HASH] = search
            search_iter
        end
        
        def append_search(search)
            search_iter = self.model.append()
            search_iter[Column::SEARCH_HASH] = search
            search_iter
        end
        
        def update(search)    
            # Append word to history
            search_iter = get_search_iter(search)
            
            search_iter = prepend_search(search) if search_iter.nil?
            
            # Select the word if it is not currently selected
            if selected_iter != search_iter
                self.selection.select_iter(search_iter)
            end
            
            # Delete a word if needed
            if self.model.nb_rows > @prefs.history_nb_rows
                self.model.remove_last
            end
            
            self.unselect_all
        end
    end
    
    class ResultListView < Gtk::TreeView
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")
    
        def initialize
            super()
            @prefs = Preferences.instance
            
            # Results : id, kanji, kana, romaji, translations
            self.model = Gtk::ListStore.new(Fixnum,String,String,String,String)
            self.selection.mode = Gtk::SELECTION_SINGLE
            self.headers_visible = false
            self.rules_hint = true
            
            columns = [
                [_("Kanji"), Column::KANJI],
                [_("Readings"), Column::READINGS],
                [_("Romaji"), Column::ROMAJI],
                [_("Meanings"), Column::MEANINGS]
            ]
            
            columns.each do |column_name, column_id|
                renderer = Gtk::CellRendererText.new
                col = Gtk::TreeViewColumn.new(column_name, renderer, :text => column_id) 
                col.sort_column_id = column_id
                col.resizable = true
                col.min_width = 40
                self.append_column(col)
            end
            
            set_visible_columns        
        end
        
        def results=(results)         
            self.model.clear
            
            results.each do |entry|
                
                row = self.model.append()
                
                row[Column::ENTRY_ID] = entry.id
                row[Column::KANJI] = entry.in_kanji.join(", ")
                row[Column::READINGS] = entry.readings_by_type.join("\n")        
                row[Column::ROMAJI] = entry.roma_readings_by_type.join("\n")            
                row[Column::MEANINGS] = entry.meanings_by_lang.join("\n")
            end
            
            self.columns_autosize
        end
    
        def set_visible_columns
            cols_prefs = [
                @prefs.col_kanji_visible,
                @prefs.col_kana_visible,
                @prefs.col_romaji_visible,
                @prefs.col_translations_visible
            ]
            
            cols = self.columns
            
            cols.each_index do |i|
                cols[i].visible = cols_prefs[i]
            end
            
            self.headers_visible = @prefs.headers_visible
        end
    
    end
    
end
end
