# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

begin 
    require 'nihongobenkyo/import'
rescue LoadError
    $stderr.puts "Can't load nihongobenkyo/import"
end

module NihongoBenkyo
module UI

    class PreferencesDialog < GladeBase 
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")

        def initialize(parent, set_visible_cols_proc, update_history_proc)
            super("preferences_dialog.glade")
            @preferences_dialog.transient_for = parent
            @prefs = Preferences.instance
            @db = Database.instance
            
            @add_dictionary_button.sensitive = defined?(NihongoBenkyo::Import) ? true : false

            # Proc
            @set_visible_cols_proc = set_visible_cols_proc
            @update_history_proc = update_history_proc
            
            init_dictionary_list_view
            
            Dictionary.list.each do |name, type, selected|
                append_dictionary(name, selected)
            end
                       
            @cols = {
                @checkbutton_headers => "headers_visible",
                @checkbutton_col_kanji => "col_kanji_visible",
                @checkbutton_col_kana => "col_kana_visible",
                @checkbutton_col_romaji => "col_romaji_visible",
                @checkbutton_col_translations => "col_translations_visible"
            }
            
            @cols.each_pair do |checkbutton, pref_name|
                checkbutton.active = @prefs.send(pref_name)
            end
            
            @history_choice.value = @prefs.history_nb_rows
        end
        
        def on_preferences_close
            @update_history_proc.call(@prefs.history_nb_rows)
            @preferences_dialog.destroy
        end
               
        def on_add_dictionary
            AddDictionaryDialog.new(self)
        end
        
        def on_remove_dictionary
            @db.close         
            selection = @dictionary_treeview.selection.selected
            
            pid = Process.fork do 
                @db.open
                dicname = selection[1]
                Dictionary.remove(dicname) 
                @db.close
                Process.kill(9, Process.pid)
            end
            
            thread = Thread.start { Process.waitpid(pid) }                       
            
            pbar = ProgressBar.new(_("Removing dictionary..."), @preferences_dialog) do 
                # Block called on click
                Process.kill(9, pid)
                thread.kill
            end
            
            pbar.signal_connect("destroy") { thread.kill }
            
            while thread.alive?
                pbar.pulse  
                Gtk.main_iteration while(Gtk.events_pending?)
                sleep 1
            end
            
            @db.open
            unless Dictionary.exists?(selection[1])
                @list_store.remove(selection)
            end
            
            pbar.destroy
        end
        
        def on_dictionary_up
            iter = @dictionary_treeview.selection.selected
            old_path = iter.path
            new_path = iter.path
            new_path.prev!
            model = @dictionary_treeview.model
            model.move_after(model.get_iter(new_path), iter)
            Dictionary.change_position(old_path, new_path)
            sensitize_buttons
        end

        def on_dictionary_down
            iter = @dictionary_treeview.selection.selected
            next_iter = iter.dup
            old_path = next_iter.path
            next_iter.next!
            new_path = next_iter.path
            @dictionary_treeview.model.move_after(iter, next_iter)
            Dictionary.change_position(old_path, new_path)
            sensitize_buttons
        end
        
        def on_column_toggled(checkbutton)
            @prefs.send("#{@cols[checkbutton]}=", 
                                      checkbutton.active?)
            @set_visible_cols_proc.call
        end
        
        def on_history_choice_value_changed
            @prefs.history_nb_rows = @history_choice.value
        end
        
        def append_dictionary(dicname,selected=1)
            row = @list_store.append()
            row[0] = selected
            row[1] = dicname
        end   
        
        def preferences_dialog
            @preferences_dialog  
        end
        
        private
               
        def init_dictionary_list_view
            @list_store = Gtk::ListStore.new(Fixnum,String)
                        
            @dictionary_treeview.model = @list_store
            @dictionary_treeview.selection.mode = Gtk::SELECTION_SINGLE
            
            renderer = Gtk::CellRendererToggle.new
            col = Gtk::TreeViewColumn.new("Active", renderer, :active => 0)
            @dictionary_treeview.append_column(col)
            
            renderer.signal_connect("toggled") do |toggled,row_iter| 
                iter = @list_store.get_iter(row_iter)
                selected = iter[0]
                dicname = iter[1]
                dic = Dictionary.new(dicname)
                if(selected == 1)
                    iter[0] = 0
                    dic.selected = 0
                else
                    iter[0] = 1
                    dic.selected = 1
                end                
            end

            renderer = Gtk::CellRendererText.new
            renderer.editable = true
            col = Gtk::TreeViewColumn.new("Dictionary", renderer, :text => 1)
            @dictionary_treeview.append_column(col) 
            
            renderer.signal_connect("edited") do |entry,row_iter,value|
                dic = Dictionary.new(@list_store.get_iter(row_iter)[1])
                dic.name = value
                @list_store.get_iter(row_iter)[1] = value
            end
       
            sensitize_buttons
            
            @dictionary_treeview.selection.signal_connect("changed") do
                sensitize_buttons
            end
        end
        
        def sensitize_buttons
            selected_iter = @dictionary_treeview.selected_iter
            if selected_iter.nil?
                @dictionary_up_button.sensitive = false
                @dictionary_down_button.sensitive = false
                @remove_dictionary_button.sensitive = false
            else
                @remove_dictionary_button.sensitive = \
                    @dictionary_treeview.has_row_selected? 
                iter_last = @list_store.get_iter((@list_store.nb_rows - 1).to_s)
                @dictionary_up_button.sensitive = \
                    ((selected_iter != @list_store.iter_first) and (not selected_iter.nil?))
                @dictionary_down_button.sensitive = \
                    ((selected_iter != iter_last) and (not selected_iter.nil?))
            end
        end
              
    end
        
end
end
